/*
 * Copyright (c) 2020 Circuit Dojo LLC
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <kernel.h>
#include <sys/byteorder.h>
#include <sys/__assert.h>
#include <sys/util.h>

#include <stdint.h>
#include <string.h>

#include <device.h>
#include <drivers/gpio.h>
#include <drivers/i2c.h>
#include <drivers/counter.h>

#include "pcf85263a.h"

#include <logging/log.h>
LOG_MODULE_REGISTER(rtc);

#define DT_DRV_COMPAT nxp_pcf85263a

struct register_map {
	uint8_t centisec; // 100th of a sec
	uint8_t sec;
	uint8_t min;
	uint8_t hour;
	uint8_t dom; // days of Month
	uint8_t dow; // days of Week
	uint8_t moncen; // month number
	uint8_t year;

	struct {
		uint8_t sec;
		uint8_t min;
		uint8_t hour;
		uint8_t date;
		uint8_t month;
	} __packed alarm1;

	struct {
		uint8_t min;
		uint8_t hour;
		uint8_t wkday;
	} __packed alarm2;
	uint8_t tsr_mode;
	// struct {
	// 	uint8_t sec;
	// 	uint8_t min;
	// 	uint8_t hour;
	// 	uint8_t dom; // days of Mont
	// 	uint8_t moncen; // month number
	// 	uint8_t year;
	// } __packed tsr1;

	// struct {
	// 	uint8_t sec;
	// 	uint8_t min;
	// 	uint8_t hour;
	// 	uint8_t dom; // days of Mont
	// 	uint8_t moncen; // month number
	// 	uint8_t year;
	// } __packed tsr2;

	// struct {
	// 	uint8_t sec;
	// 	uint8_t min;
	// 	uint8_t hour;
	// 	uint8_t dom; // days of Mont
	// 	uint8_t moncen; // month number
	// 	uint8_t year;
	// } __packed tsr3;

	// CONTROL REGISRTES
	uint8_t oscilator;
	uint8_t battery_switch;
	uint8_t pin_io;
	int8_t function;
	uint8_t intA_enable;
	uint8_t intB_enable;
	uint8_t flags;
	uint8_t stop;
	uint8_t reset;
};


struct pcf85263a_config {
	/* Common structure first because generic API expects this here. */
	struct counter_config_info generic;
	const char *bus_name;
	uint16_t addr;
};

struct pcf85263a_data
{
	struct device *pcf85263a;
	struct device *i2c;
	struct register_map registers;

	struct k_sem lock;

	/* Handlers and state when using the counter alarm API. */
	counter_alarm_callback_t counter_handler[2];
	uint32_t counter_ticks[2];

	/* Handlers and state for DS3231 alarm API. */
	pcf85263a_alarm_callback_handler_t alarm_handler[2];
	void *alarm_user_data[2];
	uint8_t alarm_flags[2];

};

static int pcf85263a_start(const struct device *dev)
{

	// Get the data pointer
	struct pcf85263a_data *data = dev->data;
	const struct pcf85263a_config *cfg = dev->config;
	data->pcf85263a = dev;

	// Stop the device
	int ret = i2c_reg_write_byte(data->i2c, cfg->addr,
								PCF85263A_STOP_EN_REG, 0); // start the timer
	if (ret)
	{
		LOG_ERR("Unable to stop RTC. (err %i)", ret);
		return ret;
	}

	return 0;
}

static int pcf85263a_stop(const struct device *dev)
{

	// Get the data pointer
	struct pcf85263a_data *data = dev->data;
	const struct pcf85263a_config *cfg = dev->config;
	data->pcf85263a = dev;


	// Write back the updated register value
	int ret = i2c_reg_write_byte(data->i2c, cfg->addr,
								PCF85263A_STOP_EN_REG, PCF85263A_STOP);
	if (ret)
	{
		LOG_ERR("Unable to stop RTC. (err %i)", ret);
		return ret;
	}

	return 0;
}

static int pcf85263a_get_value(const struct device *dev, uint32_t *ticks)
{
	return 0;
}

static int pcf85263a_set_alarm(
		const struct device *dev, uint8_t chan_id, const struct counter_alarm_cfg *alarm_cfg)
{
	ARG_UNUSED(chan_id);

	return 0;
}

static int pcf85263a_cancel_alarm(const struct device *dev, uint8_t chan_id)
{

	return 0;
}

static int pcf85263a_set_top_value(const struct device *dev, const struct counter_top_cfg *cfg)
{
	return 0;
}

static uint32_t pcf85263a_get_pending_int(const struct device *dev)
{

	return 0U;
}

static uint32_t pcf85263a_get_top_value(const struct device *dev)
{
	return 0;
}

static uint32_t pcf85263a_get_max_relative_alarm(const struct device *dev)
{
	return 0;
}

static const struct counter_driver_api pcf85263a_driver_api = {
		.start = pcf85263a_start,
		.stop = pcf85263a_stop,
		.get_value = pcf85263a_get_value,
		.set_alarm = pcf85263a_set_alarm,
		.cancel_alarm = pcf85263a_cancel_alarm,
		.set_top_value = pcf85263a_set_top_value,
		.get_pending_int = pcf85263a_get_pending_int,
		.get_top_value = pcf85263a_get_top_value,
		.get_max_relative_alarm = pcf85263a_get_max_relative_alarm
};


static const struct pcf85263a_config pcf85263a_cfg_info = {
	.generic = {
		.max_top_value = UINT32_MAX,
		.freq = 1,
		.flags = COUNTER_CONFIG_INFO_COUNT_UP,
		.channels = 2,
	},
	.bus_name = DT_INST_BUS_LABEL(0),
	.addr = DT_INST_REG_ADDR(0),
};
// ARG_UNUSED(dev);



int pcf85263a_init(const struct device *dev)
{
	struct pcf85263a_data *data = dev->data;
	const struct pcf85263a_config *cfg = dev->config;
	data->pcf85263a = dev;
	//int rc;
	
	
	/* Get the i2c device binding*/
	data->i2c = device_get_binding(cfg->bus_name);

	// Set I2C Device.
	if (data->i2c == NULL)
	{
		LOG_ERR("Failed to get pointer to %s device!",
						data->pcf85263a->name);
		return -EINVAL;
	}

	// Check if it's alive.
	uint8_t reg;
	
	int ret = i2c_reg_read_byte(data->i2c, cfg->addr,
								PCF85263A_STOP_EN_REG, &reg);
	if (ret)
	{
		LOG_ERR("Failed to read from PCF85263A! (err %i)", ret);
		return -EIO;
	}

	LOG_INF("%s is initialized!", data->pcf85263a->name);

	return 0;
}

static struct pcf85263a_data pcf85263a_data;

DEVICE_AND_API_INIT(pcf85263a, DT_INST_LABEL(0), pcf85263a_init, &pcf85263a_data,
										&pcf85263a_cfg_info, POST_KERNEL, CONFIG_I2C_INIT_PRIORITY,
										&pcf85263a_driver_api);


// DEVICE_DT_INST_DEFINE(0, ds3231_init, device_pm_control_nop, &ds3231_0_data,
// 		    &ds3231_0_config,
// 		    POST_KERNEL, CONFIG_COUNTER_MAXIM_DS3231_INIT_PRIORITY,
// 		    &ds3231_api);
